/* Stuff to use sound the Win95 way - Eric Auer 11.6.2001 */
/* this works similar to the MikMod "drv_w95.c" driver :) */

#include "ea_sound.h" /* do not forget your head(er)! */

#ifdef EA_SOUND_TIMED
	UINT myWaveTimer;
#endif

int eaSoundOpen(void) // checks for sound and opens it
{
	PCMWAVEFORMAT myWaveFormat;
	MMRESULT err; // is an unsigned int

	if (waveOutGetNumDevs() < 1) return -1;	// bad luck!
	waveOutGetDevCaps(0 /* Dev id */, &myWaveOutCaps, sizeof(WAVEOUTCAPS));
	// *** myWaveOutCaps.szPname has the driver name
	// *** ...wChannels is 1, 2, or more
	// *** ...dwFormats has bits WAVE_FORMAT_{1,2,4}{M,S}{8,16}

	hWaveGlobalRAM = GlobalAlloc(GMEM_MOVEABLE|GMEM_SHARE,
		EAWAVEBUFSIZE /* buffersize */);
	//   if it is NULL, error
	myWaveData = GlobalLock(hWaveGlobalRAM);
	//   now it will be at a fixed place in RAM

	myWaveFormat.wf.wFormatTag = WAVE_FORMAT_PCM;
	myWaveFormat.wf.nChannels = EA_WAVE_CHANNELS; // mono/stereo
	myWaveFormat.wf.nSamplesPerSec = EA_WAVE_SAMPRATE; // sampling rate
	myWaveFormat.wBitsPerSample = EA_WAVE_BITS; // resolution
	myWaveFormat.wf.nAvgBytesPerSec = 
		myWaveFormat.wf.nSamplesPerSec * myWaveFormat.wf.nChannels *
		myWaveFormat.wBitsPerSample / 8;
	myWaveFormat.wf.nBlockAlign = myWaveFormat.wBitsPerSample / 8;
	myWaveFormat.wf.nBlockAlign *= myWaveFormat.wf.nChannels;

	err = waveOutOpen(&hWaveOut,
		0 /* 0 is default, or use WAVE_MAPPER */,
		(WAVEFORMATEX *)&myWaveFormat /* obvious... */,
		0 /* optional callback function, ran as playing ends */,
		0 /* optional instance (this for C++) */,
		0 /* flags, 0 or CALLBACK_FUNCTION or so */
		);

	if (err)
	{
		// some possible values:
		// WAVERR_BADFORMAT, MMSYSERR_ALLOCATED, other
		// ^- bad sampling rate etc ^- already in use
		GlobalUnlock(hWaveGlobalRAM);
		GlobalFree(hWaveGlobalRAM);
		return (int)err;
	}

	return 0;
}

int eaSoundLoadFile(char * filename)	// guess what
{
	HFILE hWavFile;
	DWORD readBytes;
	OFSTRUCT thaFile;

	waveOutSetVolume(hWaveOut /* 0 or handle */,
		0xffffffff /* DWORD volume */
		);

	// all this would only be used for a circular buffer,
	// BUT for now, I am simply loading the whole file at
	// the beginning...
	myWaveHdr.lpData = myWaveData; // pointer to that buffer
	myWaveHdr.dwBufferLength = EAWAVEBUFSIZE;
	myWaveHdr.dwFlags = 
#ifdef EA_SOUND_TIMED
		WHDR_BEGINLOOP | WHDR_ENDLOOP; // for looping
#else
		0;
#endif
	myWaveHdr.dwLoops = 0xffffffff; // for looping
	myWaveHdr.dwUser = 0; // user defineable data


	GlobalUnlock(hWaveGlobalRAM);
	GlobalFree(hWaveGlobalRAM);
	hWavFile = OpenFile(filename, &thaFile, OF_READ);
	// see windows.h and winbase.h for OF_* ...
	// thaFile has members: szPathName nErrCode cBytes fFixedDisk ...
	if (hWavFile <= 0)
		return -1;
	hWaveGlobalRAM = GlobalAlloc(GMEM_MOVEABLE|GMEM_SHARE,
		EAWAVEBUFSIZE /* buffersize */);
	//   if it is NULL, error
	myWaveData = GlobalLock(hWaveGlobalRAM);
	//   now it will be at a fixed place in RAM

	ReadFile((HANDLE)hWavFile, myWaveData, EAWAVEBUFSIZE, &readBytes,
		NULL /* overlapped */ );

#ifdef EA_SOUND_TIMED
	waveOutPrepareHeader(hWaveOut,&myWaveHdr,sizeof(WAVEHDR));
#else
	myWaveHdr.lpData = myWaveData; /* PLUS some value! */
	myWaveHdr.dwBufferLength = readBytes; /* MINUS some value! */
	/* uhm, where is that sample rate definition stuff etc. ?? */
	waveOutPrepareHeader(hWaveOut,&myWaveHdr,sizeof(WAVEHDR));
	// at least a wave file should start like a wave header?
	// and what about that data pointer?
	// however, if everything works, we could simply use
	// myWaveData instead of &myWaveHdr :-)
#endif
	// this causes win95 to do some voodoo to prepare for playing

	myWaveCharData = myWaveData;

	// for circular buffer, we would now use
#ifdef EA_SOUND_TIMED
	timeBeginPeriod(20 /* resolution */);
	myWaveTimer = timeSetEvent(40 /* delay */, 40 /* resolution */,
		myWaveCallback /* LPTIMECALLBACK callback func */,
		NULL /* user defined dword to pass to callback */,
		TIME_PERIODIC /* kind of event */
		);
#endif

	return 0;
}

int eaSoundPlay(void)					// guess what
{
	waveOutSetVolume(hWaveOut /* 0 or handle */,
		0xffffffff /* DWORD volume */
		);
	waveOutWrite(hWaveOut,&myWaveHdr,sizeof(WAVEHDR));
	// as soon as the header is written, win95 plays
	// the wave data belonging to it in the background
	return 0;
}

long eaSoundWhere(void)	// returns current playing position
{
	MMTIME my_mmt;						// multimedia time structure
	waveOutGetPosition(hWaveOut,&my_mmt,sizeof(MMTIME));
	return (my_mmt.u.ms);
	// also possible: smpte, ticks, cb (bytes), midi, ms, sample
}

void eaSoundUnloadFile(void)			// frees RAM consumed by last file loading
{
	waveOutBreakLoop(hWaveOut);			// new idea...
	while (waveOutClose(hWaveOut) == WAVERR_STILLPLAYING)
		msec_delay(20);	// wait for end of that sound
	GlobalUnlock(hWaveGlobalRAM);		// un-pin the area :-)
	GlobalFree(hWaveGlobalRAM);			// free the RAM
	return;
}

void eaSoundClose(void)					// frees sound device
{

	// for circular buffer, we would now use
#ifdef EA_SOUND_TIMED
	timeKillEvent(myWaveTimer);
	timeEndPeriod(20);
#endif

	waveOutReset(hWaveOut);
	waveOutUnprepareHeader(hWaveOut,&myWaveHdr,sizeof(WAVEHDR));
	return;
}

