/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* WinMain(), app_main(), does all setup and cleanup  */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

#include <windows.h>
#include <windowsx.h>

#include "eyelink.h"
#include "w32_exptsppt.h"
#include "w32_demo.h"

           // the application instance: required to create windows and get resources
HANDLE application_instance = NULL;  

           // display information: size, colors, refresh rate
DISPLAYINFO dispinfo;

           // The colors of the target and background for calibration and drift correction
			// *** Was RGB(0,0,0) and RGB(128,128,128) before
COLORREF target_foreground_color = READING_TARGET_COLOR;
COLORREF target_background_color = READING_TARGET_BACK;

           // Name for experiment: goes in task bar, and in EDF file
			// *** Changed by Eric Auer as lots of other ***-marked stuff
char program_name[100] = "Windows Gaze Activated Readin Experiment";

// *** this is the big main() thing called from WinMain below
// *** new 11.6.2001 - use optional cmd line arg to override
// *** script file name from ini file
app_main(LPSTR lpCmdLine)
{
  int i, j;
  char our_file_name[260] = "TEST";

  if(open_eyelink_connection(0)) return -1;    // abort if we can't open link
  set_offline_mode();                          
  flush_getkey_queue();                        // initialize getkey() system

  get_display_information(&dispinfo);          // get window size, characteristics
  if(dispinfo.palsize==16)      // 16-color modes not functional
    {
      alert_printf("This program cannot use 16-color displays");
      goto shutdown;
    }
  else if(dispinfo.palsize)     // 256-color modes: palettes not supported by this example
    {
      alert_printf("This program is not optimized for 256-color displays");
    }

  if(make_full_screen_window(application_instance)) goto shutdown;  // create the window
  if(init_expt_graphics(full_screen_window, NULL)) goto shutdown;   // register window with EXPTSPPT

  i = SCRWIDTH/60;        // select best size for calibration target
  j = SCRWIDTH/300;       // and focal spot in target
  if(j < 2) j = 2;
  set_target_size((UINT16)i, (UINT16)j);  // tell DLL the size of target features

		// *** Was RGB(0,0,0) and RGB(128,128,128) before
  target_foreground_color = READING_TARGET_COLOR;	// color of calibration target
  target_background_color = READING_TARGET_BACK;	// background for calibration and drift correction
  set_calibration_colors(target_foreground_color, target_background_color); // tell EXPTSPPT the colors

  set_cal_sounds("", "", "");
  set_dcorr_sounds("", "off", "off");
                                     // draw a title screen
  clear_full_screen_window(target_background_color);    // clear screen
  get_new_font("Times Roman", SCRHEIGHT/32, 1);         // select a font
                                                        // Draw text
#ifdef SAMPLE_COPYRIGHT
  graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 6*SCRHEIGHT/30, 
                 "EyeLink Demonstration Experiment: Sample Code");
  graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 7*SCRHEIGHT/30,
                 "Included with the Experiment Programming Kit for Windows 95/NT");
  graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 8*SCRHEIGHT/30, 
                 "All code is Copyright (c) 1997,1998 SR Research Ltd, Toronto.");
  graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 9*SCRHEIGHT/30, 
                 "Source code may be used as template for your experiments.");
#endif

   graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 1*SCRHEIGHT/30, 
                 "EyeLink Gaze Activated Reading Experiment derived from");
  graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 2*SCRHEIGHT/30,
                 "Sample Code of the Experiment Programming Kit for Windows 95/NT");
  graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 3*SCRHEIGHT/30, 
                 "Sample Code is Copyright (c) 1997,1998 SR Research Ltd, Toronto.");
  graphic_printf(target_foreground_color, target_background_color, 1, SCRWIDTH/2, 4*SCRHEIGHT/30, 
                 "Gaze Activated Reading Code Enhancements (c) 2001 Eric Auer, Saarland.");


  i = edit_dialog(full_screen_window, "Create EDF File",            // Get the EDF file name
                 "Enter Tracker EDF file name:", our_file_name, 8);
  if(i==-1) goto shutdown;          // ALT-F4: terminate
  if(i==1)  our_file_name[0] = 0;   // Cancelled: No file name
  
  if(our_file_name[0])    // If file name set, open it
    {           
      if(!strstr(our_file_name, ".")) strcat(our_file_name, ".EDF");  // add extension
      i = open_data_file(our_file_name);                              // open file
      if(i!=0)                                                        // check for error
        {
          alert_printf("Cannot create EDF file '%s'", our_file_name);
          goto shutdown;
        }                                                              // add title to preamble
      eyecmd_printf("add_file_preamble_text 'RECORDED BY %s' ", program_name);
    }
                                      // Now configure tracker for display resolution
  eyecmd_printf("screen_pixel_coords = %ld %ld %ld %ld",    // Set display resolution   
                 dispinfo.left, dispinfo.top, dispinfo.right, dispinfo.bottom);
  eyecmd_printf("calibration_type = HV9");                  // Setup calibration type
  eyemsg_printf("DISPLAY_COORDS %ld %ld %ld %ld",           // Add resolution to EDF file
                 dispinfo.left, dispinfo.top, dispinfo.right, dispinfo.bottom);

		// SET UP TRACKER CONFIGURATION 
		      // set parser saccade thresholds (conservative settings)
  eyecmd_printf("saccade_velocity_threshold = 35");
  eyecmd_printf("saccade_acceleration_threshold = 9500");
		      // set EDF file contents 
  eyecmd_printf("file_event_filter = LEFT,RIGHT,FIXATION,SACCADE,BLINK,MESSAGE,BUTTON");
  eyecmd_printf("file_sample_data  = LEFT,RIGHT,GAZE,AREA,GAZERES");
		      // set link data (used for gaze cursor) 
  eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXATION,BUTTON");
  eyecmd_printf("link_sample_data  = LEFT,RIGHT,GAZE,AREA");
                      // Program button #5 for use in drift correction
  eyecmd_printf("button_function 5 'accept_target_fixation'");

  if(!eyelink_is_connected() || break_pressed()) goto end_expt;  // make sure we're still alive

		// RUN THE EXPERIMENTAL TRIALS (code depends on type of experiment)
                // Calling run_trials() performs a calibration followed by trials
                // This is equivalent to one block of an experiment 
                // It will return ABORT_EXPT if the program should exit
  // ***

  i = run_trials(lpCmdLine); // *** new 11.6.2001 - pass along command line
	// *** run_trials calls ReadParams to read ini file and LoadScript
	// *** to read script. new: command line can override script file
	// *** name from ini file.

  // ***

end_expt:                // END: close, transfer EDF file
  set_offline_mode();               // set offline mode so we can transfer file
  pump_delay(500);                  // delay so tracker is ready
  eyecmd_printf("close_data_file"); // close data file

  if(break_pressed()) goto shutdown;          // don't get file if we aborted experiment
  if(our_file_name[0])                        // make sure we created a file
    receive_data_file(our_file_name, "", 0);  // transfer the file, ask for a local name

shutdown:                // CLEANUP
  close_expt_graphics();           // tell EXPTSPPT to release window
  close_eyelink_connection();      // disconnect from tracker
  close_full_screen_window();
  return 0;
}


        // WinMain - Windows calls this to execute application
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  application_instance = hInstance;  // record the application instance for accessing resources
  // *** new 11.6.2001 - pass along command line args
  // *** arg is treated as an optional script file name
  // *** which overrides the value from the ini file
  app_main(lpCmdLine);               // call our real program
  close_eyelink_connection();        // make sure EYELINK DLL is released
  return 0;
} 

